
# q-expansion of newform 42.8.e.d, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 42
weight = 8
poly_data = [206496900, -316355550, 484660225, -28740, 22015, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[316355550, -484660225, 14370, -22015, 0, -1], 316355550], [[-321418780, 14370, -22015, -95, -1, 0], 968660], [[4662091145100, 3551627119700, 211009080, 160227995, 21555, 7331], 6959822100], [[2309393934300, -3538017220850, 263524245, -160709390, 0, -7331], 6959822100], [[-6980570943000, 7142478791900, -685542405, 324381935, -7185, 14684], 6959822100]]

hecke_ring_character_values = [[29, [1, 0, 0, 0, 0, 0]], [31, [0, -1, 0, 0, 0, 0]]]
aps_data = [[0, -8, 0, 0, 0, 0], [-27, 27, 0, 0, 0, 0], [0, 23, 0, 0, -1, 0], [-54, -190, -1, 1, 1, 1], [-810, 811, 6, -5, 4, -4], [-634, 1, -18, -4, -2, 1], [-8162, 8168, 2, -30, -10, -24], [7, 431, 14, -7, -32, -35], [14, -32166, 28, -14, 186, -70], [-37897, 34, -455, -136, -68, 34], [-73820, 73833, 117, -65, 91, -52], [-9, -83189, -18, 9, 384, 45], [203516, 134, 244, -536, -268, 134], [200076, -65, 80, 260, 130, -65], [-82, 41108, -164, 82, -118, 410], [-1002373, 1002395, 2455, -110, 2411, -88], [952772, -952817, -5558, 225, -5468, 180], [346, -220920, 692, -346, 4010, -1730], [-3514855, 3515144, -136, -1445, -714, -1156], [-342594, 250, 7770, -1000, -500, 250], [2289087, -2288712, -7730, -1875, -8480, -1500], [1033, -578262, 2066, -1033, -7633, -5165], [879159, -305, -1368, 1220, 610, -305], [1088, 320990, 2176, -1088, -8902, -5440], [-2829707, -1055, 4384, 4220, 2110, -1055], [2231328, -2229658, -7530, -8350, -10870, -6680], [-2551, 2010225, -5102, 2551, -34162, 12755], [-325, -10746292, -650, 325, 23318, 1625], [-11132785, 11130836, 20942, 9745, 24840, 7796], [9393108, -622, 14796, 2488, 1244, -622], [15849639, 5987, 18227, -23948, -11974, 5987], [2327, -15008198, 4654, -2327, -950, -11635], [-22499838, 22512638, -43266, -64000, -68866, -51200], [-24698016, 4689, -37522, -18756, -9378, 4689], [14276, 13239254, 28552, -14276, -4666, -71380], [-34842989, 34843843, 73131, -4270, 71423, -3416], [6289534, -6275896, -78756, -68190, -106032, -54552], [-14102, 23803430, -28204, 14102, -31070, 70510], [-37498942, -932, 62728, 3728, 1864, -932], [4060, -69534522, 8120, -4060, 112938, -20300], [8095342, -8113016, 27584, 88370, 62932, 70696], [17879514, 9623, -92044, -38492, -19246, 9623], [-13322, 24275326, -26644, 13322, 6520, 66610], [-83981823, 83969881, -72344, 59710, -48460, 47768], [-626486, 17660, -194122, -70640, -35320, 17660], [55244120, -55273164, -122416, 145220, -64328, 116176], [-44740434, -49118, -258916, 196472, 98236, -49118], [54075425, -34146, 14907, 136584, 68292, -34146], [91085938, -91066571, 246146, -96835, 207412, -77468], [74671, 70210699, 149342, -74671, -104778, -373355], [-30152, -51300436, -60304, 30152, 116990, 150760], [90491460, -7596, -190698, 30384, 15192, -7596], [-7405618, 7436877, -698674, -156295, -761192, -125036], [-64314801, 1365, 186840, -5460, -2730, 1365], [-44282, 52787716, -88564, 44282, 707398, 221410], [315951542, -315975094, -251846, 117760, -204742, 94208], [62272089, -62359249, -597477, 435800, -423157, 348640], [-52862, 221060151, -105724, 52862, 649265, 264310], [1956539, -1924854, 400274, -158425, 336904, -126740], [-15489046, -20532, 1333066, 82128, 41064, -20532], [196777597, -196756124, 855498, -107365, 812552, -85892], [356443327, 30226, 758285, -120904, -60452, 30226], [102285546, 187605, -9298, -750420, -375210, 187605], [-11316966, 11311172, 157764, 28970, 169352, 23176], [-128662, 369604529, -257324, 128662, 338356, 643310], [-52982, 317764999, -105964, 52982, 1061497, 264910], [26287, 447269411, 52574, -26287, 1102954, -131435], [-1277442927, 26036, -139672, -104144, -52072, 26036], [558611724, -558460932, -1426956, -753960, -1728540, -603168], [260473330, -164408, 22672, 657632, 328816, -164408], [-347020122, 347158524, -523476, -692010, -800280, -553608], [183862, 671835396, 367724, -183862, -669276, -919310], [962593252, -962619853, 109227, 133005, 162429, 106404], [-74147, 468589765, -148294, 74147, 1523790, 370735], [-916670118, -185313, -1360828, 741252, 370626, -185313], [46298, -327868308, 92596, -46298, 3065964, -231490], [329024116, -328921158, -92806, -514790, -298722, -411832], [-57331, 1323188761, -114662, 57331, -1382478, 286655], [-388724, 377899588, -777448, 388724, -2042000, 1943620], [297847411, -297932965, -3567852, 427770, -3396744, 342216], [29513972, -400066, 1749328, 1600264, 800132, -400066], [-91352928, -306637, 4844354, 1226548, 613274, -306637], [215573998, -215810624, 3078764, 1183130, 3552016, 946504], [-1245843600, 45991, 331412, -183964, -91982, 45991], [165486, 1578184427, 330972, -165486, 1461829, -827430], [641141, -427587120, 1282282, -641141, 1244850, -3205705], [1000175672, 45472, -3837254, -181888, -90944, 45472], [295978, -518339453, 591956, -295978, 3310956, -1479890], [319720842, 485924, -4856142, -1943696, -971848, 485924], [-374385830, 666667, -1534520, -2666668, -1333334, 666667], [92658, -2268033658, 185316, -92658, -1580692, -463290], [-1244826742, 1245766254, -3837578, -4697560, -5716602, -3758048], [-1647018604, 1646969607, 1304339, 244985, 1402333, 195988], [2147953563, -169767, -7034148, 679068, 339534, -169767], [205869, -2985606477, 411738, -205869, 1535156, -1029345], [1854853328, 353576, 3979654, -1414304, -707152, 353576], [107608, -3580987751, 215216, -107608, -2281823, -538040], [-604606812, 604490586, 10925364, 581130, 11157816, 464904], [-460615, 852775209, -921230, 460615, 7922048, 2303075], [-149053, -1087879883, -298106, 149053, -6825900, 745265], [1817587400, -503306, 7386618, 2013224, 1006612, -503306], [-1013155053, 1011902849, 4350333, 6261020, 6854741, 5008816], [-2199428306, 2198511479, -9462202, 4584135, -7628548, 3667308], [1109800, -798438224, 2219600, -1109800, -2115182, -5549000], [-1843470803, 1842857796, 12501186, 3065035, 13727200, 2452028], [-1808904787, 1808432951, -13675036, 2359180, -12731364, 1887344], [604014241, -1547473, -811108, 6189892, 3094946, -1547473], [-1721860, -1156798528, -3443720, 1721860, -206842, 8609300], [-1263903400, 1262633366, -5192144, 6350170, -2652076, 5080136], [-1694557205, 393128, 6896020, -1572512, -786256, 393128], [-1731283, -265530812, -3462566, 1731283, 11338461, 8656415], [-958961832, 959799884, 14134258, -4190260, 12458154, -3352208], [1799576906, 322032, 7513936, -1288128, -644064, 322032], [-1378277737, 1378890296, -2769166, -3062795, -3994284, -2450236], [2859139221, 979090, -19615237, -3916360, -1958180, 979090], [3311407336, -3309958354, -18095788, -7244910, -20993752, -5795928], [-4828590676, 936909, 8089230, -3747636, -1873818, 936909], [-2784188736, 2782070706, -3831150, 10590150, 404910, 8472120], [-957336, -4828388145, -1914672, 957336, 14953275, 4786680], [185841616, -1880308, -6421960, 7521232, 3760616, -1880308], [-1640601033, 1642067832, -2357420, -7333995, -5291018, -5867196], [-604356591, -526178, -10613248, 2104712, 1052356, -526178], [-237766, 6413212445, -475532, 237766, 2354903, 1188830], [2737400636, -2737669147, -8496650, 1342555, -7959628, 1074044], [822307, 1887096851, 1644614, -822307, -6650786, -4111535], [-3327925465, 1685218, -7940747, -6740872, -3370436, 1685218], [1005424, 6046810178, 2010848, -1005424, 12772408, -5027120], [-3359316, -1217224782, -6718632, 3359316, -14782044, 16796580], [2740551633, -416811, 15685193, 1667244, 833622, -416811], [-214957, 7958824201, -429914, 214957, -3524100, 1074785], [-5058022205, 5058350664, -9588192, -1642295, -10245110, -1313836], [-1428960410, 2572432, 11855312, -10289728, -5144864, 2572432], [-2738089, 3447137638, -5476178, 2738089, 25433565, 13690445], [-4026029314, -2029646, 40694622, 8118584, 4059292, -2029646], [-314310, 13647640666, -628620, 314310, 31219330, 1571550], [5623978801, 1400350, 30054472, -5601400, -2800700, 1400350], [-9385976768, 9386693142, 19361762, -3581870, 17929014, -2865496], [-5955537612, 5957106650, 17332096, -7845190, 14194020, -6276152], [7647889689, -409962, 1975563, 1639848, 819924, -409962], [5395991006, -5398880324, -25822928, 14446590, -20044292, 11557272], [7761549954, -1468644, -25858138, 5874576, 2937288, -1468644], [2191332, -9274695707, 4382664, -2191332, -37255475, -10956660], [-14565028592, 14569764184, -23688000, -23677960, -33159184, -18942368], [-7724653825, -330097, -34714868, 1320388, 660194, -330097], [1178471245, -1178906618, -15751356, 2176865, -14880610, 1741492], [4299088504, -96466, -20085526, 385864, 192932, -96466], [4400190464, 807663, -3579468, -3230652, -1615326, 807663], [-9452146480, 9450370874, 27078892, 8878030, 30630104, 7102424], [1353526, -24890561872, 2707052, -1353526, -16574564, -6767630], [1042364, -6371140238, 2084728, -1042364, -10436192, -5211820], [-3563282, 4870248430, -7126564, 3563282, 51471060, 17816410], [7272075262, -3605588, 47794802, 14422352, 7211176, -3605588], [14145678210, 1337287, -78328468, -5349148, -2674574, 1337287], [-845796, 2048636208, -1691592, 845796, -28325394, 4228980], [-2045298891, 2039992034, -40566752, 26534285, -29953038, 21227428], [4875129774, -369480, -27206370, 1477920, 738960, -369480], [-5128053, 212424795, -10256106, 5128053, -43858210, 25640265], [-2758756, 7184993042, -5517512, 2758756, 7889792, 13793780], [12745481315, -960636, 38082720, 3842544, 1921272, -960636], [10926409297, -10927848375, -50590099, 7195390, -47711943, 5756312], [-1064522, -18739456986, -2129044, 1064522, 46054092, 5322610], [-25720757682, -3458942, -11457036, 13835768, 6917884, -3458942], [11171037181, -416419, -43715603, 1665676, 832838, -416419], [3531911, -26175487204, 7063822, -3531911, 22705850, -17659555], [28424537542, -28421233118, -44199628, -16522120, -50808476, -13217696], [12962279020, -12958531508, -35768944, -18737560, -43263968, -14990048], [-16621361532, 16622702701, 43922821, -6705845, 41240483, -5364676], [-26043655463, 26039683880, -27367962, 19857915, -19424796, 15886332]]
